==> seplib-6.3.9.tar.gz: seplib-6.3.9/seplib_base/lib/corelibs/superset/sect_assign.c <==
#define SET_SDOC 1
#include<sitedef.h>
#include<math.h>
#include<superset.h>
#include<superset_internal.h>

_XFUNCPROTOBEGIN
int SEPDFF_assign_best(sep_dff *info, int *reassign, int *nassign)
_XFUNCPROTOEND
{
  char **machnew,**machold,**machnew_unique,**machold_unique,**mach_new_unique;
  char **dffnew,temp_ch[1024];
  int nold,nnew;
  int i,*mach_count,*thread_count,*ncount_old,*ncount_new,j;
  int *assign_sect,*assign_thread;
  int ihigh,ilow;
  int  *mach_old_new,*compress_loc_new,*compress_loc_old;
  int found,extra;
  int istart;


  if(info->master_data==1) istart=0;
  else istart=1;




  machnew=(char**) malloc(sizeof(char*)*info->nmpi);
  dffnew=(char**) malloc(sizeof(char*)*info->nmpi);
  for(i=0; i < info->nmpi; i++){
    machnew[i]=malloc(sizeof(char)*1024);
    dffnew[i]=malloc(sizeof(char)*1024);
  }


  if(0!=SEPDFF_grab_mpi_names(info,machnew,dffnew)){
    for(i=0; i < info->nmpi; i++) free(machnew[i]); free(machnew);
    for(i=0; i < info->nmpi; i++) free(dffnew[i]); free(dffnew);
    return (sepwarn(NOT_MET,"tag:%s trouble getting mpi_names \n", info->name));
  }
 for(i=0; i < info->nmpi; i++) free(dffnew[i]); free(dffnew);

  machnew_unique=(char**) malloc(sizeof(char*)*info->nmpi);
  for(i=0; i < info->nmpi; i++) machnew_unique[i]=malloc(sizeof(char)*1024);
  

  compress_loc_new=(int*)malloc(sizeof(int)*info->nmpi);
  if(0!=sep3d_find_unique_names(info->nmpi,machnew,&nnew,machnew_unique,
     compress_loc_new,istart)){
    for(i=0; i < info->nmpi; i++) free(machnew_unique[i]); free(machnew_unique);
    for(i=0; i < info->nmpi; i++) free(machnew[i]); free(machnew);
    free(compress_loc_new);
  } 
  
  machold_unique=(char**) malloc(sizeof(char*)*info->nsect);
  for(i=0; i < info->nsect; i++) machold_unique[i]=malloc(sizeof(char)*1024);
  compress_loc_old=(int*)malloc(sizeof(int)*info->nsect);


  if(0!=sep3d_find_unique_names(info->nsect,info->mach_sect,&nold,machold_unique,compress_loc_old,0)){
    for(i=0; i < info->nmpi; i++) free(machnew_unique[i]); free(machnew_unique);
    for(i=0; i < info->nsect; i++) free(machold_unique[i]); free(machold_unique);
    for(i=0; i < info->nmpi; i++) free(machnew[i]); free(machnew);
    free(compress_loc_old); free(compress_loc_new);
  } 

  assign_thread=(int*) malloc(sizeof(int)*info->nmpi);
  assign_sect=(int*) malloc(sizeof(int)*info->nsect);
  for(i=0;i < info->nmpi;i++) assign_thread[i]=0;
  for(i=0;i < info->nsect;i++) assign_sect[i]=-1;


  ihigh=ceil((float)info->nsect/((float)(info->nmpi-istart)));
  ilow=floor((float)info->nsect/((float)(info->nmpi-istart)));
  extra=info->nsect-ilow*(info->nmpi-istart);

  /* machine number (in parsed old list) that goes with the machine number in parsed new list*/
  mach_old_new=(int*) malloc(sizeof(int)*nold);
  for(j=0; j < nold; j++){
    for(i=0, mach_old_new[j]=-1; i < nnew; i++){
      if(0==strcmp(machnew_unique[i],machold_unique[j])) mach_old_new[j]=i;
    }
  }

  for(i=0; i < info->nmpi; i++) free(machnew_unique[i]); free(machnew_unique);
  for(i=0; i < info->nsect; i++) free(machold_unique[i]); free(machold_unique);

  /*we will begin by filling up to the minimum (ilow) on each processor  that are on the same machines*/
  for(i=0; i < info->nsect; i++){
    /*see if the machine that the section was on is being used */
    if(mach_old_new[compress_loc_old[i]]!=-1){
      /*try to find a thread that is < ilow  to assign this section to*/
      j=istart; found=-1;
      while(j < info->nmpi && found==-1){
        if(assign_thread[j] < ilow && mach_old_new[compress_loc_old[i]] == compress_loc_new[j]) found=j;
        j++;
      }
      if(found!=-1){
        assign_sect[i]=found;
        assign_thread[found]++;
      }
    }
  }
  
  /*we will begin by filling up to the maximum (ihigh) on each processor  that are on the same machines*/
  for(i=0; i < info->nsect; i++){
    if(extra>0 && assign_sect[i]==-1){
       /*see if the machine that the section was on is being used */
       if(mach_old_new[compress_loc_old[i]]!=-1){
         /*try to find a thread that is < ihigh  to assign this section to*/
         j=istart; found=-1;
         while(j < info->nmpi && found==-1){
           if(assign_thread[j] < ihigh && mach_old_new[compress_loc_old[i]] == compress_loc_new[j]) found=j;
           j++;
         }
         if(found!=-1){
           assign_sect[i]=found;
           assign_thread[found]++;
           extra--;
         }
       }
     }
  }
  free(compress_loc_new); free(compress_loc_old); free(mach_old_new);
 *nassign=0;
 /*now assign everything else*/
 for(i=0; i < info->nsect; i++){
    if(assign_sect[i]==-1){
      found=-1; j=istart;
      while(j < info->nmpi && found==-1){
        if(assign_thread[j] < ilow || (assign_thread[j] < ihigh && extra>0)) found=j;
         else j++;
      }
      if(j==info->nmpi && found==-1){
         seperr("bug in reasignment \n");
      }
      assign_thread[j]++; assign_sect[i]=j;
      if(assign_thread[j] >ilow) extra--;
      reassign[*nassign]=i; *nassign+=1;
   }
   if( 0!=SEPDFF_set_mach_sect(info,i,machnew[assign_sect[i]]) ){
     
     return(sepwarn(NOT_MET,"trouble setting tag and mach sect for tag %s \n",
      info->name));
   }
 }
  for(i=0; i < info->nmpi; i++) free(machnew[i]); free(machnew);

 if(0!=SEPDFF_set_sect_threads(info,assign_sect)){
   free(assign_sect); free(assign_thread);
   return(sepwarn(NOT_MET,"trouble setting sect_threads for tag %s \n",info->name));
 }
 free(assign_sect);free(assign_thread);

 if(0!=SEPDFF_set_axis_thread_info(info))
   return(sepwarn(NOT_MET,"trouble calculating sect_ithread tag=%s \n",info->name));

 return(SUCCESS); 

}



_XFUNCPROTOBEGIN
int sep3d_assign_sect(char *tag, int impi,int sect)
_XFUNCPROTOEND
{
sep_3d *info;
char temp_ch[1204],dffpath[1024];

info = tag_info_sep3d(tag, INQUIRE);  /* get info on this tag */
if(info==SEPNULL)
  return(sepwarn(NOT_MET,"non-initialized structure tag=%s \n",tag));
if(info->dff==SEPNULL)
  return(sepwarn(NOT_MET,"tag is not distributed tag=%s \n",tag));

if(0!=sep3d_grab_mach_name(temp_ch,impi))
  return(sepwarn(NOT_MET,"trouble grabbing machine name \n"));
   if( 0!=SEPDFF_set_mach_sect(info->dff,sect-1,temp_ch))
     return(sepwarn(NOT_MET,"trouble setting tag and mach sect for tag %s \n",
      info->name));

 if(0!=SEPDFF_set_sect_thread(info->dff,sect-1,impi)){
   return(sepwarn(NOT_MET,"trouble setting sect_thread for tag %s \n",info->name));
 }
  if(0!=sep3d_grab_thread_prefix(dffpath,impi))
    return(sepwarn(NOT_MET,"trouble grabbing thread prefix \n"));
  if(0!=SEPDFF_make_dff_name(info->dff,temp_ch,dffpath,tag,sect-1,impi))
    return(sepwarn(NOT_MET,"trouble making dff name  \n"));
  if(0!=SEPDFF_set_tag_sect(info->dff,sect-1,temp_ch))
    return(sepwarn(NOT_MET,"trouble setting tag name \n"));
 

 if(0!=SEPDFF_set_axis_thread_info(info->dff))
   return(sepwarn(NOT_MET,"trouble calculating sect_ithread tag=%s \n",info->name));
 return(SUCCESS); 
  


}

_XFUNCPROTOBEGIN
int sep3d_find_unique_names(int nlist, char **list, int *norig, char**orig, int *mapping,int istart)
_XFUNCPROTOEND
{
  int i,j,found;

 
  *norig=0;
  for(j=istart; j < nlist; j++){
    for(i=0,mapping[j]=-1; i < *norig; i++){
      if(0==strcmp(orig[i], list[j])){
        mapping[j]=i;
      }
    }
    if(mapping[j]==-1){
      strcpy(orig[*norig],list[j]); mapping[j]=*norig; *norig=*norig+1;
    }
  }
  return(SUCCESS);
}

==> seplib-6.3.12.tar.gz: seplib-6.3.12/seplib_base/lib/corelibs/superset/sect_assign.c <==
#define SET_SDOC 1
#include<sitedef.h>
#include<math.h>
#include<superset.h>
#include<superset_internal.h>

_XFUNCPROTOBEGIN
int SEPDFF_assign_best(sep_dff *info, int *reassign, int *nassign)
_XFUNCPROTOEND
{
  char **machnew,**machold,**machnew_unique,**machold_unique,**mach_new_unique;
  char **dffnew,temp_ch[1024];
  int nold,nnew;
  int i,*mach_count,*thread_count,*ncount_old,*ncount_new,j;
  int *assign_sect,*assign_thread;
  int ihigh,ilow;
  int  *mach_old_new,*compress_loc_new,*compress_loc_old;
  int found,extra;
  int istart;


  if(info->master_data==1) istart=0;
  else istart=1;




  machnew=(char**) malloc(sizeof(char*)*info->nmpi);
  dffnew=(char**) malloc(sizeof(char*)*info->nmpi);
  for(i=0; i < info->nmpi; i++){
    machnew[i]=malloc(sizeof(char)*1024);
    dffnew[i]=malloc(sizeof(char)*1024);
  }


  if(0!=SEPDFF_grab_mpi_names(info,machnew,dffnew)){
    for(i=0; i < info->nmpi; i++) free(machnew[i]); free(machnew);
    for(i=0; i < info->nmpi; i++) free(dffnew[i]); free(dffnew);
    return (sepwarn(NOT_MET,"tag:%s trouble getting mpi_names \n", info->name));
  }
 for(i=0; i < info->nmpi; i++) free(dffnew[i]); free(dffnew);

  machnew_unique=(char**) malloc(sizeof(char*)*info->nmpi);
  for(i=0; i < info->nmpi; i++) machnew_unique[i]=malloc(sizeof(char)*1024);
  

  compress_loc_new=(int*)malloc(sizeof(int)*info->nmpi);
  if(0!=sep3d_find_unique_names(info->nmpi,machnew,&nnew,machnew_unique,
     compress_loc_new,istart)){
    for(i=0; i < info->nmpi; i++) free(machnew_unique[i]); free(machnew_unique);
    for(i=0; i < info->nmpi; i++) free(machnew[i]); free(machnew);
    free(compress_loc_new);
  } 
  
  machold_unique=(char**) malloc(sizeof(char*)*info->nsect);
  for(i=0; i < info->nsect; i++) machold_unique[i]=malloc(sizeof(char)*1024);
  compress_loc_old=(int*)malloc(sizeof(int)*info->nsect);


  if(0!=sep3d_find_unique_names(info->nsect,info->mach_sect,&nold,machold_unique,compress_loc_old,0)){
    for(i=0; i < info->nmpi; i++) free(machnew_unique[i]); free(machnew_unique);
    for(i=0; i < info->nsect; i++) free(machold_unique[i]); free(machold_unique);
    for(i=0; i < info->nmpi; i++) free(machnew[i]); free(machnew);
    free(compress_loc_old); free(compress_loc_new);
  } 

  assign_thread=(int*) malloc(sizeof(int)*info->nmpi);
  assign_sect=(int*) malloc(sizeof(int)*info->nsect);
  for(i=0;i < info->nmpi;i++) assign_thread[i]=0;
  for(i=0;i < info->nsect;i++) assign_sect[i]=-1;


  ihigh=ceil((float)info->nsect/((float)(info->nmpi-istart)));
  ilow=floor((float)info->nsect/((float)(info->nmpi-istart)));
  extra=info->nsect-ilow*(info->nmpi-istart);

  /* machine number (in parsed old list) that goes with the machine number in parsed new list*/
  mach_old_new=(int*) malloc(sizeof(int)*nold);
  for(j=0; j < nold; j++){
    for(i=0, mach_old_new[j]=-1; i < nnew; i++){
      if(0==strcmp(machnew_unique[i],machold_unique[j])) mach_old_new[j]=i;
    }
  }

  for(i=0; i < info->nmpi; i++) free(machnew_unique[i]); free(machnew_unique);
  for(i=0; i < info->nsect; i++) free(machold_unique[i]); free(machold_unique);

  /*we will begin by filling up to the minimum (ilow) on each processor  that are on the same machines*/
  for(i=0; i < info->nsect; i++){
    /*see if the machine that the section was on is being used */
    if(mach_old_new[compress_loc_old[i]]!=-1){
      /*try to find a thread that is < ilow  to assign this section to*/
      j=istart; found=-1;
      while(j < info->nmpi && found==-1){
        if(assign_thread[j] < ilow && mach_old_new[compress_loc_old[i]] == compress_loc_new[j]) found=j;
        j++;
      }
      if(found!=-1){
        assign_sect[i]=found;
        assign_thread[found]++;
      }
    }
  }
  
  /*we will begin by filling up to the maximum (ihigh) on each processor  that are on the same machines*/
  for(i=0; i < info->nsect; i++){
    if(extra>0 && assign_sect[i]==-1){
       /*see if the machine that the section was on is being used */
       if(mach_old_new[compress_loc_old[i]]!=-1){
         /*try to find a thread that is < ihigh  to assign this section to*/
         j=istart; found=-1;
         while(j < info->nmpi && found==-1){
           if(assign_thread[j] < ihigh && mach_old_new[compress_loc_old[i]] == compress_loc_new[j]) found=j;
           j++;
         }
         if(found!=-1){
           assign_sect[i]=found;
           assign_thread[found]++;
           extra--;
         }
       }
     }
  }
  free(compress_loc_new); free(compress_loc_old); free(mach_old_new);
 *nassign=0;
 /*now assign everything else*/
 for(i=0; i < info->nsect; i++){
    if(assign_sect[i]==-1){
      found=-1; j=istart;
      while(j < info->nmpi && found==-1){
        if(assign_thread[j] < ilow || (assign_thread[j] < ihigh && extra>0)) found=j;
         else j++;
      }
      if(j==info->nmpi && found==-1){
         seperr("bug in reasignment \n");
      }
      assign_thread[j]++; assign_sect[i]=j;
      if(assign_thread[j] >ilow) extra--;
      reassign[*nassign]=i; *nassign+=1;
   }
   if( 0!=SEPDFF_set_mach_sect(info,i,machnew[assign_sect[i]]) ){
     
     return(sepwarn(NOT_MET,"trouble setting tag and mach sect for tag %s \n",
      info->name));
   }
 }
  for(i=0; i < info->nmpi; i++) free(machnew[i]); free(machnew);

 if(0!=SEPDFF_set_sect_threads(info,assign_sect)){
   free(assign_sect); free(assign_thread);
   return(sepwarn(NOT_MET,"trouble setting sect_threads for tag %s \n",info->name));
 }
 free(assign_sect);free(assign_thread);

 if(0!=SEPDFF_set_axis_thread_info(info))
   return(sepwarn(NOT_MET,"trouble calculating sect_ithread tag=%s \n",info->name));

 return(SUCCESS); 

}



_XFUNCPROTOBEGIN
int sep3d_assign_sect(char *tag, int impi,int sect)
_XFUNCPROTOEND
{
sep_3d *info,*infot;
char temp_ch[1204],dffpath[1024],sect_name[1024];

info = tag_info_sep3d(tag, INQUIRE);  /* get info on this tag */


if(info==SEPNULL)
  return(sepwarn(NOT_MET,"non-initialized structure tag=%s \n",tag));
if(info->dff==SEPNULL)
  return(sepwarn(NOT_MET,"tag is not distributed tag=%s \n",tag));

if(sect < 1 || sect > info->dff->nsect)
  return(sepwarn(NOT_MET,"invalid sect %d assign must be between %d and %d \n",
   sect,1,info->dff->nsect));
   

if(0!=sep3d_grab_mach_name(temp_ch,impi))
  return(sepwarn(NOT_MET,"trouble grabbing machine name \n"));
   if( 0!=SEPDFF_set_mach_sect(info->dff,sect-1,temp_ch))
     return(sepwarn(NOT_MET,"trouble setting tag and mach sect for tag %s \n",
      info->name));
 if(0!=SEPDFF_set_sect_thread(info->dff,sect-1,impi)){
   return(sepwarn(NOT_MET,"trouble setting sect_thread for tag %s \n",info->name));
 }
 SEPDFF_sect_name(info->dff,sect-1,sect_name);

if( 0!=SEPDFF_set_tag_sect(info->dff,sect-1,sect_name))
   return(sepwarn(NOT_MET,"trouble setting tag and mach sect for tag %s \n",
    info->name));


 if(impi == info->dff->impi){
    if(0!= SEP3D_make_section(info,sect-1,impi))
     return(sepwarn(NOT_MET,"trouble making section %d for tag %s \n",
      sect-1,info->name));
 }
                                                                                      
 if( info->dff->nmpi >1){
       if(0!=sep3d_broadcast_headers(sect_name,
        info->dff->sect_thread[sect-1],0))
        return(sepwarn(NOT_MET,"trouble broadcasting headers \n"));
  }


  if(0!=sep3d_grab_thread_prefix(dffpath,impi))
    return(sepwarn(NOT_MET,"trouble grabbing thread prefix \n"));
  if(0!=SEPDFF_make_dff_name(info->dff,temp_ch,dffpath,tag,sect-1,impi))
    return(sepwarn(NOT_MET,"trouble making dff name  \n"));
  if(0!=SEPDFF_set_tag_sect(info->dff,sect-1,temp_ch))
    return(sepwarn(NOT_MET,"trouble setting tag name \n"));
 

 if(0!=SEPDFF_set_axis_thread_info(info->dff))
   return(sepwarn(NOT_MET,"trouble calculating sect_ithread tag=%s \n",info->name));
 return(SUCCESS); 
  


}

_XFUNCPROTOBEGIN
int sep3d_find_unique_names(int nlist, char **list, int *norig, char**orig, int *mapping,int istart)
_XFUNCPROTOEND
{
  int i,j,found;

 
  *norig=0;
  for(j=istart; j < nlist; j++){
    for(i=0,mapping[j]=-1; i < *norig; i++){
      if(0==strcmp(orig[i], list[j])){
        mapping[j]=i;
      }
    }
    if(mapping[j]==-1){
      strcpy(orig[*norig],list[j]); mapping[j]=*norig; *norig=*norig+1;
    }
  }
  return(SUCCESS);
}
